#! /bin/sh

# Neuros OSD Internet Radio Player
# By wkrik@yahoo.com 1/09 v1.3
# Source: http://www.ugu.com/software/iradio
#
# This script will go to shoutcast, search the selected genre and
# display a list of stations to choose from. Once selected, it will
# clear your playquee, add the station's URL and start streaming
# the station with xmms2 on your OSD.
#
# Requires: wget
#
# Release V1.3: YAFI: Added HTTP variable incase shoutcast changes URL
#		Added DEBUG option to see if shoutcast data is downloading
# Release V1.2: YAFI: Display the (n)ext list of stations
#		From Genre Menu, "r" refreshes cache on ALL genres
#		Search for unique stations on shoutcast from genre menu
#		  (ie. beatles, scorpions, love, etc...)
#		Time calculator added to show how long caching will take
#		  Based on NUMSTAT and the number of GENRES
#		Command prompt changed. (To many commands now)
#               Help command added for growing list of commands.
#
# Release V1.1: YAFI: Added ability to (s)top listening by command
#		Added ability to (r)efresh stations with "r" command
#               Added caching of stations to speed displaying
#
# Release V1.0: Initial release 
#

# Debug: Set debug to equal 1 to see if collecting from Shoutcast
DEBUG=""

# HTTP support to control the URL if shoutcast changes it.
HTTP="http://classic.shoutcast.com"

# Display how many stations at a time on the screen
DISP=10

# Download how many stations per genre. The OSD is slow, 
# 100 stations will take up to 80 seconds to download and process
# for each genre. if you have 10 genres that's 30 minutes to process.
# but you should only have to do it once.
NUMSTAT=20

# TMP is a temp directory used for storing cached stations 
# and other tmp data. he following should be fully qualified:
TMP="/mnt/tmpfs/mount_USB/zsrc/tmp/"

XMMS="/usr/local/bin/xmms2"
WGET="/mnt/tmpfs/mount_USB/Applications/wget"

# "list" is the list of menu items displayed and genres that are searched
# on shoutcast. YOU can add any genre that can be searched on shoutcast
# Syntax: menu_number:genre_search_pattern
# NOTE: Do not use "spaces" (11:Rock and Roll), Use "plus" (11:Rock+and+Roll)
#
list="
1:70s
2:80s
3:Ambient
4:Blues
5:Clasical
6:Jazz
7:Opera
8:Pop
9:Rock
10:Techno
"

#
# -------- START OF SCRIPT -------
#

verify_setup()
{
  if [ ! -d "$TMP" ]; then
    echo; echo "Creating TMP directory...";echo
    mkdir -p $TMP
  fi
  if [ ! -f "$WGET" ]; then
    echo; echo "ERROR: Cannot find 'wget' program at: $WGET"; echo
    exit
  fi
  if [ ! -f "$XMMS" ]; then
    echo; echo "ERROR: Cannot find 'xmms2' program at: $XMMS"; echo
    exit
  fi
}


# Calculate the amount of time to cache stations
calc_cache()
{
 # TM = time to process 10 stations
 TM=8
 totaltime=`expr $TM "*" $NUMSTAT / 10`
 echo; echo -n "Process a single genre: $totaltime seconds "
 sectime=""
 if [ $totaltime -gt 60 ]; then
   sectime=`expr $totaltime / 60`
   mintime=`expr $totaltime % 60`
   echo "= $sectime.$mintime mintues"
 else 
   echo
 fi
 if [ "$select" = "r" ]; then
   atotaltime=`expr $totaltime "*" $totalgenres`
   echo -n "Process all genres:     $atotaltime seconds "
   sectime=""
   if [ $atotaltime -gt 60 ]; then
     sectime=`expr $atotaltime / 60`
     mintime=`expr $atotaltime % 60`
     echo "= $sectime.$mintime mintues"
   else
     echo
   fi
 fi
 echo  
}




help_genre()
{
cat <<EOF


 Genre Menu Commands are:
 
 * Enter Genre # from the list
 * Enter a search pattern to search for specific stations
 * Press [enter] by itself to exit the program
 * "h" will display this help message
 * "s" will stop playing music if a station is playing
 * "r" will refresh station cache on ALL genres
     NOTE: If you have NUMSTAT=100 it will take up to 80 
     seconds to scan each genre for 100 stations.
     
EOF

  echo -n "Press [enter] to continue..."
  read dummy
}


help_stations()
{
cat <<EOF


 Station Menu Commands:
 
 * Select a station number to play
 * Press [enter] by itself to return to the genre menu
 * "h" will display this help message
 * "n" will display the next greoup of stations in the list collected
 * "r" will refresh the list of stations in cache
 * "s" will stop playing music if a station is playing
     
EOF

  echo -n "Press [enter] to continue..."
  read dummy
}


collect_stations()
{
    # GET THE AVAILABLE STATIONS FROM THE SELECTED GENRE
    echo -n "  Collecting $gen Stations...$"
      $WGET -qO $TMP  \
"$HTTP/directory/index.phtml?sgenre=${gen}&maxbitrate=9999&numresult=${NUMSTAT}"
  if [ -n $DEBUG ]; then
    echo "$HTTP/directory/index.phtml?sgenre=${gen}&maxbitrate=9999&numresult=${NUMSTAT}"
  fi
    echo -n "$"
    grep -E 'listlinks|tunein' $TMP > $TMP.new
    echo -n "$"
  
    rm $TMP
    cnt=-1
    x=0
    cat $TMP.new | while read line
    do
      if [ $x -eq 0 ]; then 
        URL=`echo $line | cut -d"=" -f4- | cut -d"\"" -f2- | cut -d"\"" -f1`
        URL="${HTTP}${URL}"
        if [ -n $DEBUG ]; then
 	  echo "URL = $URL"
        fi
        x=1
        cnt=`expr $cnt + 1`
#        echo $cnt
      else
        TTL=`echo $line | sed s/\<a/\~/ | cut -d"~" -f2- | cut -d">" -f2- | cut -d"<" -f1 | cut -c1-70`
     	if [ -z "$TTL" ]; then 
          cnt=`expr $cnt - 1`
        else
          echo "$cnt:$URL:$TTL" >> $TMP.$select
          echo -n "$"
        fi
        x=0
      fi
    done
}


cache_all()
{
    echo
    for line in $list
    do
      select=`echo $line | cut -d":" -f1`
      gen=`echo $line | cut -d":" -f2`
      #echo "$select ---- $gen"
      echo -n "$select of $totalgenres:"
      rm -f $TMP.$select
      collect_stations
      echo
    done 

}

stop_music()
{
    echo; echo "  Stopping Music..."
    $XMMS stop &
    sleep 3
    boo="1"
}


genre_menu()
{
  select="s"
  totalgenres=0
  boo="1"
  while [ "$boo" = "1" ]
  do
    echo
    echo " Neuros OSD Internet Radio"
    echo " -------------------------"
    
    # GET THE LIST OF GENRE'S TO SELECT
    for line in $list
    do
      line=`echo $line | sed 's/:/ : /g' | sed 's/+/ /g'`
      echo "  $line" 
      totalgenres=`expr $totalgenres + 1`
    done 
  
    echo; echo -n "  Command (h=help) :" 
    read select
    boo=""
    if [ -z "$select" ]; then
      echo
      exit 0
    fi
    case "$select" in
    'r')
      calc_cache
      cache_all
      boo="1"
      ;;
    'h')
      help_genre
      boo="1"
      ;;
    's')
      stop_music
      boo="1"
    esac
  done

  for line in $list
  do
    gen=`echo $line | grep ^"${select}:" | cut -d":" -f2`
    if [ -n "$gen" ]; then
      echo; echo "  $gen station selected..."; echo
      break
    fi
    if [ -n "$select" ]; then
      echo; echo "  Searching for: $gen"; echo
      gen="$select"
      break
    fi
  done
}


listen_station()
{
  if  [ -n "$station" ]; then
    LINE=`grep ^"${station}:" $TMP.$select`
    sURL=`echo $LINE | cut -d":" -f2-3`
    DESC=`echo $LINE | cut -d":" -f4-`
    
    #echo "$sURL  -- $DESC"
    $WGET -qO $TMP.pls "$sURL"
    
    
    pURL=`grep "File1=" $TMP.pls | cut -d"=" -f2-`
    
    echo
    echo "Starting Radio Station..."
      $XMMS stop
      $XMMS clear
      $XMMS add $pURL
      echo
      echo "*** PLAYING: $DESC"
      $XMMS play
  fi
    echo
    rm -f $TMP $TMP.new $TMP.pls 
}



select_stations()
{
  leave=""
  Brec=0; 
  Erec=`expr $Brec + $DISP`
  while [ -z "$leave" ]
  do

  if [ ! -f "$TMP.$select" ]; then
    collect_stations
    echo; echo
  fi
    
    cnt=0
    echo 
    cat $TMP.$select | cut -d":" -f1,4- | cut -c1-45 | while read list
    do
     cnt=`expr $cnt + 1`
     if [ $cnt -ge $Brec -a $cnt -le $Erec ]; then
        echo $list
     fi
     if [ $cnt -gt $Erec ]; then 
       break
     fi

    done 
  
    # SELECT A STATION
    echo; echo -n "  Command (h=help) :"
    read station
 
    if [ -z $station ]; then
      break
    fi

    case "$station" in
    's')
       STOP="$select"; stop_music
       ;;
    'n')
      leave=""
      Brec=`expr $Erec + 1`
      if [ $Erec -gt $NUMSTAT ]; then 
        Brec=0
      fi
      Erec=`expr $Brec + $DISP`
      ;;
    'r')
      echo
      calc_cache
      rm -f $TMP.$select
      ;;
    'h')
      help_stations
      ;;
    *)
     listen_station
     #leave="yes"
     ;;
    esac
  done
}



verify_setup
TMP="$TMP/iradio.tmp"
while [ 1 ]
do
  genre_menu
  select_stations
done
exit 0
